/**
 * Copyright (c) 2020 Nectar Inc
 * All rights reserved.
 * 
 */

#include "app_error.h"
#include "app_util_platform.h"
#include "boards.h"
#include "nrf_delay.h"
#include "nrf_drv_clock.h"
#include "nrf_drv_gpiote.h"
#include "nrf_gpio.h"


#include "nectar_skyy.h"

#if NECTAR_UART_ONLY
#include "nectar_uart.h"
#else
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#endif


//flag to make sure we only init once
static bool gpio_init = false;

//accel interrupt indicators
volatile bool lis2dh12_accel_intr1 = false;
volatile bool lis2dh12_accel_intr2 = false;
volatile bool nectar_pour_detected = false;

void lis2dh12_int1_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action) {
  lis2dh12_accel_intr1 = true;
  //if (!nectar_button_pressed()) {
    //NRF_LOG_RAW_INFO("\rINT1");
  //} else {
    //NRF_LOG_RAW_INFO("\r\n*********************Interrupt by Accel but button is pressed!\r\n");
  //}
}

void lis2dh12_int2_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action) {
  lis2dh12_accel_intr2 = true;
  //if (lis2dh12_accel_intr1) {
    //nectar_pour_detected = true;
    //NRF_LOG_RAW_INFO("\rINT2!");
  //} else {
    //NRF_LOG_RAW_INFO("\rIGNORING INT2!");

  //}
}

#if NECTAR_GG_RTC_PRESENT
void rtc_3028_c7_int_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action) {
  NRF_LOG_RAW_INFO("\r\n***********************Interrupt up by RTC!\r\n");
  rtc_wakeup_int = true;
}
#endif

void nectar_gpio_init(void) {
  ret_code_t err_code;

  if (gpio_init) {
    return;
  }

  err_code = nrf_drv_gpiote_init();
  APP_ERROR_CHECK(err_code);

  nrf_drv_gpiote_in_config_t in_config1 = GPIOTE_CONFIG_IN_SENSE_LOTOHI(false);
  in_config1.pull = NRF_GPIO_PIN_NOPULL;
  in_config1.hi_accuracy = false;
  err_code = nrf_drv_gpiote_in_init(LIS_INT1_PIN, &in_config1, lis2dh12_int1_handler);
  APP_ERROR_CHECK(err_code);

#if NECTAR_GG_RTC_PRESENT
  nrf_drv_gpiote_in_config_t in_config2 = GPIOTE_CONFIG_IN_SENSE_HITOLO(false);
  in_config2.pull = NRF_GPIO_PIN_NOPULL;
  in_config2.hi_accuracy = false;
  err_code = nrf_drv_gpiote_in_init(RTC_INT_PIN, &in_config2, rtc_3028_c7_int_handler);
  APP_ERROR_CHECK(err_code);
#endif 

  nrf_drv_gpiote_in_config_t in_config3 = GPIOTE_CONFIG_IN_SENSE_LOTOHI(false);
  in_config3.pull = NRF_GPIO_PIN_NOPULL; //NRF_GPIO_PIN_PULLUP;
  in_config3.hi_accuracy = false;
  err_code = nrf_drv_gpiote_in_init(LIS_INT2_PIN, &in_config3, lis2dh12_int2_handler);
  APP_ERROR_CHECK(err_code);
  gpio_init = true;
}

void nectar_gpio_intr_enable(void) {
  nrf_drv_gpiote_in_event_enable(LIS_INT1_PIN, true);
  nrf_drv_gpiote_in_event_enable(LIS_INT2_PIN, true);
#if NECTAR_GG_RTC_PRESENT
  nrf_drv_gpiote_in_event_enable(RTC_INT_PIN, true);
#endif
}

void nectar_gpio_intr_enable_by_pin(nrfx_gpiote_pin_t pin) {
  nrf_drv_gpiote_in_event_enable(pin, true);
}

void nectar_gpio_intr_disable(void) {
  nrf_drv_gpiote_in_event_disable(LIS_INT1_PIN);
  nrf_drv_gpiote_in_event_disable(LIS_INT2_PIN);
#if NECTAR_GG_RTC_PRESENT
  nrf_drv_gpiote_in_event_disable(RTC_INT_PIN);
#endif
}

void nectar_gpio_intr_disable_by_pin(nrfx_gpiote_pin_t pin) {
  nrf_drv_gpiote_in_event_disable(pin);
}