/*
 * Copyright (c) 2020 Nectar Inc
 * All_Rights_Reserved.
 * 
 */

#include "nrf.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"
#include "nrf_delay.h"

#include "nrf_saadc.h"

#include "nectar_skyy.h"
#include "nectar_gpio.h"
#include "nectar_saadc.h"

#if NECTAR_UART_ONLY
#include "nectar_uart.h"
#else
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#endif

static nrf_saadc_value_t      adc_buffer[MAX_ADC_SAMPLES];
static bool                   saadc_init = false;
static bool                   analog_ready = true;

void nectar_envelope_adc_read(void)
{
    if ((saadc_init) && (analog_ready))
    {
        //Analog circuit enable is active low
        nrf_gpio_pin_clear(DRV_ANALOG);

        nrf_saadc_task_trigger(NRF_SAADC_TASK_START);

        while(!nrf_saadc_event_check(NRF_SAADC_EVENT_STARTED));
        nrf_saadc_event_clear(NRF_SAADC_EVENT_STARTED);

        nrf_saadc_task_trigger(NRF_SAADC_TASK_STOP);
        while(!nrf_saadc_event_check(NRF_SAADC_EVENT_STOPPED));
        nrf_saadc_event_clear(NRF_SAADC_EVENT_STOPPED);

#if NECTAR_ENVADC_DEBUG
        for (int loop = 0; loop < MAX_ADC_SAMPLES; loop++)
        {
            if ((adc_buffer[loop] > 40) || (adc_buffer[loop] < -40))
              NRF_LOG_RAW_INFO("%d, %d,\n", loop, adc_buffer[loop]);
        }
#endif
        analog_ready = false;
    }
}

void nectar_envelope_adc_clear(void)
{
#if NECTAR_ENVADC_DEBUG
    NRF_LOG_RAW_INFO("%s %d\n", __FUNCTION__, __LINE__);
#endif
    if (saadc_init)
    {
        //Analog circuit enable is active low
        nrf_gpio_pin_set(DRV_ANALOG);
        memset(adc_buffer, 0, MAX_ADC_SAMPLES);
        analog_ready = true;
    }
}

void nectar_envelope_adc_init(void)
{
#if NECTAR_ENVADC_DEBUG
    //NRF_LOG_RAW_INFO("%s %d\n", __FUNCTION__, __LINE__);
    NRF_LOG_RAW_INFO("\radc init");
    //NRF_LOG_FLUSH();
#endif
    if (!saadc_init)
    {
        ret_code_t err_code;
        nrf_saadc_channel_config_t channel_config =
            NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(ADC_INPUT);

        channel_config.gain =     NRF_SAADC_GAIN1_3;
        channel_config.acq_time = NRF_SAADC_ACQTIME_3US;
        channel_config.mode =     NRF_SAADC_MODE_SINGLE_ENDED;

        memset(adc_buffer, 0, MAX_ADC_SAMPLES);

        //Analog circuit enable is active low, set to high on init
        nrf_gpio_cfg_output(DRV_ANALOG);
        nrf_gpio_pin_set(DRV_ANALOG);

        nrf_saadc_enable();
        nrf_saadc_channel_init(ADC_INPUT, &channel_config);
        nrf_saadc_buffer_init(adc_buffer, MAX_ADC_SAMPLES);

        saadc_init = true;
#if NECTAR_ENVADC_DEBUG
        NRF_LOG_RAW_INFO("\rADC channel %d", ADC_CHANNEL); 
        NRF_LOG_RAW_INFO("\rMax adc period %d",MAX_ADC_PERIOD);
        NRF_LOG_RAW_INFO("\rMax adc samples %d",MAX_ADC_SAMPLES);
        NRF_LOG_RAW_INFO("\rADC gain %d", channel_config.gain);
        NRF_LOG_RAW_INFO("\rADC acquisition time %d", channel_config.acq_time);
        NRF_LOG_RAW_INFO("\rADC mode %d", channel_config.mode);
        //NRF_LOG_FLUSH();
#endif
    }
}